library(MHTdiscrete)
library(reshape2)
library(ggplot2)

source("attainpval.R")

### simulation to calculate FWER and Power
B <- 2000; alpha <- 0.05;
M <- c(5, 10, 15); pi0 <-c(0.2, 0.4, 0.6, 0.8);
N <- 25*c(1:6)
MHolm.FWER <- numeric(); MHolm.Pow <- numeric()
Holm.FWER <- numeric(); Holm.Pow <- numeric()
TH.FWER <- numeric(); TH.Pow <- numeric()
MHolm.FWER.pi0 <- MHolm.Pow.pi0 <- Holm.FWER.pi0 <- Holm.Pow.pi0 <- TH.FWER.pi0 <- TH.Pow.pi0  <-numeric()

set.seed(1001)
for(l in 1:length(M)){
  for(k in 1:length(pi0)){
    for(a in 1:length(N)){
      h0 <- M[l]*pi0[k]; h1<-M[l]-h0;
      Pval <- matrix(0,B,M[l]); P.list <- list();
      MHolm.adjp <- matrix(0,B,M[l]); MHolm.R <- matrix(0,B,1); MHolm.V <- matrix(0,B,1)
      MHolm.total.FWER <- 0; MHolm.total.Pow <- 0
      Holm.adjp <- matrix(0,B,M[l]); Holm.R <- matrix(0,B,1); Holm.V <- matrix(0,B,1)
      Holm.total.FWER <- 0; Holm.total.Pow <- 0
      TH.adjp <- matrix(0,B,M[l]); TH.R <- matrix(0,B,1); TH.V <- matrix(0,B,1)
      TH.total.FWER <- 0; TH.total.Pow <- 0;   
      
      for (i in 1:B){
        theta1 <- rep(0.1,M[l]) ;
        theta2 <- c(rep(0.1,h0),rep(0.2,h1))
        X1 <- rbinom(M[l],N[a],theta1); X2 <- rbinom(M[l],N[a],theta2)
        P.list[[i]] <- list(NaN *seq(M[l]))
        for (j in 1:M[l]){
          P.list[[i]][[j]] <- hyperpset(N[a],N[a],X1[j]+X2[j])
          Pval[i,j] <- P.list[[i]][[j]][X1[j]+1]
        }
        #### Using Modified Bonferroni Procedure (Proc 1)
        MHolm.adjp[i,] <- MHolm.p.adjust(Pval[i,], P.list[[i]])
        MHolm.R[i] <- sum(MHolm.adjp[i,] <= alpha)
        MHolm.V[i] <- sum(MHolm.adjp[i,1:h0] <= alpha)
        
        MHolm.total.FWER <- MHolm.total.FWER + ifelse(MHolm.V[i]>0,1,0)
        MHolm.total.Pow <- MHolm.total.Pow +ifelse((MHolm.R[i]-MHolm.V[i])>0,1,0)
        
        ####Compare with the original Bonferroni procedure
        Holm.adjp[i,] <- p.adjust(Pval[i,], method = "holm")
        Holm.R[i] <- sum(Holm.adjp[i,] <= alpha)
        Holm.V[i] <- sum(Holm.adjp[i,1:h0] <= alpha)
        
        Holm.total.FWER <- Holm.total.FWER + ifelse(Holm.V[i]>0,1,0)
        Holm.total.Pow <- Holm.total.Pow +ifelse((Holm.R[i]-Holm.V[i])>0,1,0)
        
        ####Compare with the Modified Tarone-Holm procedure
        TH.adjp[i,] <- TH.p.adjust(Pval[i,], P.list[[i]])
        TH.R[i] <- sum(TH.adjp[i,] <= alpha)
        TH.V[i] <- sum(TH.adjp[i,1:h0] <= alpha)
        
        TH.total.FWER <- TH.total.FWER + ifelse(TH.V[i]>0,1,0)
        TH.total.Pow <- TH.total.Pow +ifelse((TH.R[i]-TH.V[i])>0,1,0)
     }
      
      MHolm.FWER[a] <- MHolm.total.FWER/B; MHolm.Pow[a] <- MHolm.total.Pow/B
      Holm.FWER[a] <- Holm.total.FWER/B; Holm.Pow[a] <- Holm.total.Pow/B
      TH.FWER[a] <- TH.total.FWER/B; TH.Pow[a] <- TH.total.Pow/B
    }
    
    MHolm.FWER.pi0 <- append(MHolm.FWER.pi0, MHolm.FWER)
    Holm.FWER.pi0 <- append(Holm.FWER.pi0, Holm.FWER)
    TH.FWER.pi0 <- append(TH.FWER.pi0, TH.FWER)
    MHolm.Pow.pi0 <- append(MHolm.Pow.pi0, MHolm.Pow)
    Holm.Pow.pi0 <- append(Holm.Pow.pi0, Holm.Pow)
    TH.Pow.pi0 <- append(TH.Pow.pi0, TH.Pow)
    
  }
}
outcome <- data.frame(cbind(No=c(1:(length(M)*length(pi0)*length(N))), M=rep(M,each=length(pi0)*length(N)), pi0=rep(rep(pi0, each=length(N)),length(M)), N=rep(N,length(M)*length(pi0)),
                            MHolm.FWER=MHolm.FWER.pi0,
                            TH.FWER=TH.FWER.pi0,
                            Holm.FWER=Holm.FWER.pi0,
                            MHolm.Pow=MHolm.Pow.pi0,
                            TH.Pow=TH.Pow.pi0,
                            Holm.Pow=Holm.Pow.pi0))
## Plotting Simulated FWER
FWER <- melt(outcome[,2:7],id.vars = c( "M", "pi0","N"), variable.name = "Methods",value.name = "level")
FWER$M <- factor(FWER$M,levels=M,labels=c("M=5","M=10", "M=15"))
FWER$pi0 <- factor(FWER$pi0,levels=pi0,labels = c("pi0=0.2", "pi0=0.4", "pi0=0.6","pi0=0.8"))
FWER$Methods <- factor(FWER$Methods,levels=c("MHolm.FWER","TH.FWER","Holm.FWER"),
                       labels=c("MHolm","Tarone-Holm","Holm"))

qplot(N, level, data=FWER, shape=Methods, color=Methods, facets=M~pi0, # geom = "line", #main = "FWER Comparisons for Step-up Procedures",
      xlab="N", ylab="Simulated FWER") +
  theme_bw() +
  geom_line(aes(linetype=Methods,color=Methods))+
  theme(legend.position="top")

## Plotting Simulated Power
Pow <- melt(outcome[,c(2:4,8:10)],id.vars = c( "M", "pi0","N"), variable.name = "Methods",value.name = "level")
Pow$M <- factor(Pow$M,levels=M, labels=c("M=5","M=10","M=15"))
Pow$pi0 <- factor(Pow$pi0,levels=pi0,labels = c("pi0=0.2", "pi0=0.4", "pi0=0.6","pi0=0.8"))
Pow$Methods <- factor(Pow$Methods,levels=c("MHolm.Pow","TH.Pow","Holm.Pow"),
                      labels=c("MHolm","Tarone-Holm","Holm"))

qplot(N, level, data=Pow, shape=Methods, color=Methods, facets=M~pi0, 
      xlab="N", ylab="Simulated Minimal Power") +
  theme_bw() +
  geom_line(aes(linetype=Methods,color=Methods))+
  theme(legend.position="top")
