library(MHTdiscrete)
library(reshape2)
library(ggplot2)

source("attainpval.R")

B <- 2000; alpha <- 0.05; mu0 <- 2; mu1 <-10
M <- c(5,10,15); pi0 <- c(0.2,0.4,0.6,0.8) ; rho <- c(0:9)*0.1;

MHoch.FWER <- numeric(); MHoch.Pow <- numeric()
Hoch.FWER <- numeric(); Hoch.Pow <- numeric()
Roth.FWER <- numeric(); Roth.Pow <- numeric()

MHoch.FWER.pi0 <- Hoch.FWER.pi0 <- Roth.FWER.pi0 <- MHoch.Pow.pi0 <- Hoch.Pow.pi0 <- Roth.Pow.pi0 <- numeric()


set.seed(1001)

for(l in 1:length(M)){
  for(k in 1:length(pi0)){
    for(a in 1:length(rho)){
      Pval <- matrix(0,B,M[l]); P.list <- list();
      MHoch.adjp <- matrix(0,B,M[l]); MHoch.R <- matrix(0,B,1); MHoch.V <- matrix(0,B,1)
      MHoch.total.FWER <- 0; MHoch.total.Pow <- 0
      Hoch.adjp <- matrix(0,B,M[l]); Hoch.R <- matrix(0,B,1); Hoch.V <- matrix(0,B,1)
      Hoch.total.FWER <- 0; Hoch.total.Pow <- 0
      Roth.adjp <- matrix(0,B,M[l]); Roth.R <- matrix(0,B,1); Roth.V <- matrix(0,B,1)
      Roth.total.FWER <- 0; Roth.total.Pow <- 0
      h0 <- M[l]*pi0[k]; h1 <- M[l]-h0
      
      for (i in 1:B){
        theta1 <- rep(mu0,M[l]) ;
        theta2 <- c(rep(mu0,round(h0)),rep(mu1,round(h1)))
        p <- c(rep(mu0,round(h0)),rep(mu1,round(h1)))
        X11 <- rpois(M[l], (1-rho[a])*theta1); X10 <- rpois(1, rho[a]*mu0)
        X21 <- rpois(M[l], (1-rho[a])*theta2); X20 <- c(rep(rpois(1, rho[a]*mu0), round(h0)),
                                                        rep(rpois(1, rho[a]*mu1), round(h1)))
        X1 <- X11+X10; X2 <- X21+X20
        P.list[[i]] <- list(NaN *seq(M[l]))
        for (j in 1:M[l]){
          P.list[[i]][[j]] <- binpset(X1[j],X1[j]+X2[j],0.5)
          Pval[i,j] <- P.list[[i]][[j]][X1[j]+1]
        }
        #### Using Modified Holme Procedure (Proc 2)
        MHoch.adjp[i,] <- MHoch.p.adjust(Pval[i,], P.list[[i]])
        MHoch.R[i] <- sum(MHoch.adjp[i,] <= alpha)
        MHoch.V[i] <- sum(MHoch.adjp[i,1:h0] <= alpha)
        
        MHoch.total.FWER <- MHoch.total.FWER + ifelse(MHoch.V[i]>0,1,0)
        MHoch.total.Pow <- MHoch.total.Pow +ifelse((MHoch.R[i]-MHoch.V[i])>0,1,0)
        
        ####Compare with the original Hochberg procedure
        Hoch.adjp[i,] <- p.adjust(Pval[i,], method = "hochberg")
        Hoch.R[i] <- sum(Hoch.adjp[i,] <= alpha)
        Hoch.V[i] <- sum(Hoch.adjp[i,1:h0] <= alpha)
        
        Hoch.total.FWER <- Hoch.total.FWER + ifelse(Hoch.V[i]>0,1,0)
        Hoch.total.Pow <- Hoch.total.Pow +ifelse((Hoch.R[i]-Hoch.V[i])>0,1,0)
        
        Roth.adjp[i,] <- Roth.p.adjust(Pval[i,], P.list[[i]])
        Roth.R[i] <- sum(Roth.adjp[i,] <= alpha)
        Roth.V[i] <-  sum(Roth.adjp[i,1:h0] <= alpha)
        
        Roth.total.FWER <- Roth.total.FWER + ifelse(Roth.V[i]>0,1,0)
        Roth.total.Pow <- Roth.total.Pow +ifelse((Roth.R[i]-Roth.V[i])>0,1,0)
        
        
      }
      
      MHoch.FWER[a] <- MHoch.total.FWER/B; MHoch.Pow[a] <- MHoch.total.Pow/B
      Hoch.FWER[a] <- Hoch.total.FWER/B; Hoch.Pow[a] <- Hoch.total.Pow/B
      Roth.FWER[a] <- Roth.total.FWER/B; Roth.Pow[a] <- Roth.total.Pow/B
    }
    
    
    MHoch.FWER.pi0 <- append(MHoch.FWER.pi0, MHoch.FWER)
    Hoch.FWER.pi0 <- append(Hoch.FWER.pi0, Hoch.FWER)
    Roth.FWER.pi0 <- append(Roth.FWER.pi0, Roth.FWER)
    MHoch.Pow.pi0 <- append(MHoch.Pow.pi0, MHoch.Pow)
    Hoch.Pow.pi0 <- append(Hoch.Pow.pi0, Hoch.Pow)
    Roth.Pow.pi0 <- append(Roth.Pow.pi0, Roth.Pow)
  }
}
outcome <- data.frame(cbind(No=c(1:(length(M)*length(pi0)*length(rho))), M=rep(M,each=length(pi0)*length(rho)), pi0=rep(rep(pi0, each=length(rho)),length(M)), rho=rep(rho,length(M)*length(pi0)),
                            MHoch.FWER=MHoch.FWER.pi0,
                            Roth.FWER=Roth.FWER.pi0,
                            Hoch.FWER=Hoch.FWER.pi0,
                            MHoch.Pow=MHoch.Pow.pi0,
                            Roth.Pow=Roth.Pow.pi0,
                            Hoch.Pow=Hoch.Pow.pi0))

## Plotting Simulated FWER
FWER <- melt(outcome[,2:7],id.vars = c("M", "pi0","rho"), variable.name = "Methods",value.name = "level")
FWER$M <- factor(FWER$M,levels=M, labels=c("M=5","M=10", "M=15"))
FWER$pi0 <- factor(FWER$pi0,levels=pi0,labels = c("pi0=0.2", "pi0=0.4", "pi0=0.6","pi0=0.8"))
FWER$Methods <- factor(FWER$Methods,levels=c("MHoch.FWER","Roth.FWER","Hoch.FWER"),
                       labels=c("MHoch","Roth","Hochberg"))

qplot(rho, level, data=FWER, shape=Methods, color=Methods, facets=M~pi0, # geom = "line", #main = "FWER Comparisons for Step-up Procedures",
      xlab=expression(rho), ylab="Simulated FWER")+
  theme_bw() +
  geom_line(aes(linetype=Methods,color=Methods))+
  theme(legend.position="top")

## Plotting Simulated Power
Pow <- melt(outcome[,c(2:4,8:10)],id.vars = c( "M", "pi0","rho"), variable.name = "Methods",value.name = "level")
Pow$M <- factor(Pow$M,levels=M, labels=c("M=5","M=10","M=15"))
Pow$pi0 <- factor(Pow$pi0,levels=pi0,labels = c("pi0=0.2", "pi0=0.4", "pi0=0.6","pi0=0.8"))
Pow$Methods <- factor(Pow$Methods,levels=c("MHoch.Pow","Roth.Pow","Hoch.Pow"),
                      labels=c("MHoch","Roth","Hochberg"))

qplot(rho, level, data=Pow, shape=Methods, color=Methods, facets=M~pi0, # geom = "line", #main = "Average Power Comparisons for Step-up Procedures",
      xlab=expression(rho), ylab="Simulated Minimal Power") +
  theme_bw() +
  geom_line(aes(linetype=Methods,color=Methods))+
  theme(legend.position="top")

